#!/bin/sh
# TWM Menu Maker Installer
# ©2024 J. S. Gilstrap - All Rights Reserved.
# & GPL ≥2.1  https://www.gnu.org/license

# DEFAULTS
# Set DeskTop Width & Height.
DskTpWd=1280 ; DskTpHt=800

# Set TWM Config home.
# Where system.twmrc is kept.
TWMhome=/etc/X11/twm

# For executables and SystemD
# setup or another running method.
Exe=/usr/local/bin

# System V InitD directory & commands. (RedHat specific?)
 InitD=/etc/rc.d/init.d
ChkCnf="chkconfig twm-menu-maker on"
  Srvc="service   twm-menu-maker start"

#------------------------ End Settings -----------------------------------------
# Check for TWM in your path.
which twm > /dev/null 2>&1 || \
{ echo "TWM is not installed or not in your path. Quitting." ; exit 1 ; }

# Read Line Echo function. A Cat Here File without a system call.
RLE(){ local IFS=$'' ; while read line ; do echo "$line" ; done ; }

# Config file backup function.
BkUpRC()
	{
	local orig bkup
	if [ "$UID" = 0 ]
	then
		orig=${1}/system.twmrc ; bkup=${1}/system-twmrc.mmtwm
	else
		orig=${1}/.twmrc ; bkup=${1}/.twmrc-mmtwm
	fi
	if [ ! -f "$bkup" -a -f "$orig" ]
	then
		mv $orig $bkup && \
		printf "\nThe original ${orig##*/} has been backed up to ${bkup##*/} ... " || \
		printf "\nFailed to BkUp ${orig##*/} ... "
	fi
	}

# Make executable directory function.
mkExeDir()
	{ 
	if [ ! -d "$1" ]
	then
		mkdir -p $1
		if [ $? = 0 ]
		then 
			printf "\nThe directory $1 has been created.\nMake sure it is part of your path ... "
		else
			printf "\nFailed to make $1 directory. Quitting."
			exit 1
		fi
	fi
	}

# Install files function.
nstlFls()
	{
	[ "$#" -lt 2 ] && return 1
	local home=$1 exe=$2
	printf "\nFiles: TWM-Help.txt, setbgcolor"
	cp TWM-Help.txt $home ; cp setbgcolor $exe

	if [ $(uname -s) = Linux ]
	then
		cp tmprtr $exe
		printf ", tmprtr & twm-menu-maker ... "
	else
		printf " & twm-menu-maker ... "
	fi

	[ -n "$3" ] && exe=$3
	sed -e "s+#TWMhome#+${home}+" -e "s+#DskTpWd#+${DskTpWd}+" -e "s+#DskTpHt#+${DskTpHt}+" \
	twm-menu-maker.in > ${exe}/twm-menu-maker 2> /dev/null && chmod 755 ${exe}/twm-menu-maker 
	return $?
	}

printf "\n[1mTWM Menu Maker Installation Script[0m\n"

# Set desktop size.
unset dtwd dtht ; yn=y
printf "\nSet desktop size:\n"
while :
do
	printf "Width [${DskTpWd}] : "
	read dtwd ; [ -n "$dtwd" ] && DskTpWd=$dtwd
	printf "Height [${DskTpHt}] : "
	read dtht ; [ -n "$dtht" ] && DskTpHt=$dtht
	printf "Resolution: [1m${DskTpWd}×${DskTpHt}[0m\n"
	printf "Is this OK? [$yn] : "
	read isOK
	echo
	[ -n "$isOK" ] && yn=$isOK
	[ ${yn,} = y ] && break
done

# User install.
if [ "$UID" -ne 0 ]
then
	[ -z "$HOME" ] && { echo "Home directory not set. Quitting." ; exit 1 ;}
	printf "Installing as normal user in $HOME ... "
	# Backup original .twmrc file & make Exe directory.
	BkUpRC $HOME ; mkExeDir ${HOME}/bin
	# Install files and generate .twmrc
	nstlFls $HOME ${HOME}/bin && ${HOME}/bin/twm-menu-maker
	if [ $? = 0 ]
	then
RLE << EoF
Done.

[1mtwm-menu-maker[0m has been placed in ${HOME}/bin and ran 
to create[1m .twmrc[0m in $HOME . When new programs have
been installed run [1mtwm-menu-maker[0m again to update menu. 

EoF
		exit
	else
		echo Failed.
		exit 1
	fi
fi

# System wide install.
unset pro ; yn=n
echo   "Running this as root for system wide install may be buggy. It is"
echo   "initially recomended to run this as a regular user for single install."
printf "Proceed? [$yn] : "
read pro
[ -n "$pro" ] && yn=$pro ; [ ${yn,} != y ] && exit 1

# Is System V?
unset sysv ; SysV=n
printf "\nUse System V Init instead of SystemD or another method? [$SysV] : "
read sysv
[ -n "$sysv" ] && SysV=$sysv

# Set path defaults.
unset twmh exe initd ; yn=y
echo
while :
do
	printf "  TWM Home [${TWMhome}] : "
	read twmh
	[ -n "$twmh" ] && TWMhome=$twmh

	printf "  Exe Path [${Exe}] : "
	read exe
	[ -n "$exe" ] && Exe=$exe

	if [ ${SysV,} = y ]
	then
		printf " InitD Dir [${InitD}] : "
		read initd
		[ -n "$initd" ] && InitD=$initd
	fi

	printf "\n  TWM Home : ${TWMhome}\n"
	printf   "  Exe Path : ${Exe}\n"
	[ ${SysV,} = y ] && \
	printf   "InitD Path : ${InitD}\n"

	printf "Is this OK? [$yn] : "
	read isOK
	[ -n "$isOK" ] && yn=$isOK
	[ ${yn,} = y ] && break
done

# Backup original system.twmrc file & make Exe directory.
BkUpRC $TWMhome ; mkExeDir $Exe
# Install.
if [ ${SysV,} = y ]
then
	printf "\nSetting up for System V Init ... "
	# Install files and generate system.twmrc
	nstlFls $TWMhome $Exe $InitD && $ChkCnf && $Srvc
	if [ $? = 0 ]
	then
RLE << EoF
Done.

[1mtwm-menu-maker[0m has been placed in ${InitD}, the service 
setup, started and created[1m system.twmrc[0m in $TWMhome

EoF
	else
		echo Failed.
	fi
else
	printf "\nSetting up for SystemD running or another method ... "
	# Install files and generate system.twmrc
	nstlFls $TWMhome $Exe && ${Exe}/twm-menu-maker
	if [ $? = 0 ]
	then
RLE << EoF 
Done.

[1mtwm-menu-maker[0m has been placed in $Exe
and run to create[1m system.twmrc[0m in $TWMhome

Create a twm-menu-maker.service file for SystemD or
if TWM is only used through vncserver them adding 
it to its .service file is another option. It can
also be run manually, only needed when programs
are installed or uninstalled.

EoF
	else
		echo Failed.
	fi
fi

